package tech.riemann.ims.controller.platform.dictionary;

import java.util.List;

import org.nutz.lang.Lang;
import org.nutz.lang.Strings;
import org.springframework.http.HttpStatus;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import club.zhcs.lina.starter.exception.BizException;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import tech.riemann.ims.entity.dictionary.Dictionary;
import tech.riemann.ims.entity.dictionary.Group;
import tech.riemann.ims.service.dictionary.IDictionaryService;
import tech.riemann.ims.service.dictionary.IGroupService;

/**
 * <p>
 * 码本数据 前端控制器
 * </p>
 *
 * @author Kerbores(kerbores@gmail.com)
 *
 * @since 2024-07-22 13:56:55
 */
@RestController
@Tag(name = "Dictionary", description = "码本数据")
@RequiredArgsConstructor
public class DictionaryController {

    private final IDictionaryService dictionaryService;

    private final IGroupService groupService;

    @GetMapping("groups")
    @Operation(summary = "分页查询码本分组")
    public IPage<Group> groups(
                               @Parameter(description = "页码") @RequestParam(required = false, defaultValue = "1") int page,
                               @Parameter(description = "页面大小") @RequestParam(required = false, defaultValue = "10") int size,
                               @Parameter(description = "搜索关键词") @RequestParam(required = false, defaultValue = "") String key) {
        boolean hasLike = Strings.isNotBlank(key);
        key = String.format("%%%s%%", key);
        return groupService.page(Page.<Group> of(page, size),
                                 Wrappers.<Group> lambdaQuery()
                                         .like(hasLike, Group::getDescription, key)
                                         .orderByAsc(Group::getId));
    }

    @GetMapping("group/{id}")
    @Operation(summary = "码本分组详情")
    public Group groupDetail(@Parameter(description = "码本分组id", required = true) @PathVariable long id) {
        return groupService.getById(id);
    }

    @PutMapping("group")
    @Operation(summary = "增加/编辑码本分组")
    public Group saveOrUpdateGroup(@Validated @Parameter(description = "码本分组") @RequestBody Group group) {
        if (group.getId() != null && group.getId() > 0) {
            if (groupService.update(group, Wrappers.<Group> lambdaUpdate().eq(Group::getId, group.getId()))) {
                return group;
            } else {
                throw BizException.create("更新码本分组失败!");
            }
        } else {
            if (groupService.save(group)) {
                return group;
            } else {
                throw BizException.create("保存码本分组失败!");
            }
        }
    }

    @DeleteMapping("group/{key}")
    @Operation(summary = "删除码本分组")
    @ResponseStatus(HttpStatus.OK)
    public void deleteGroup(@Parameter(description = "码本分组key", required = true) @PathVariable String key) {
        if (groupService.remove(Wrappers.<Group> lambdaQuery().eq(Group::getKey, key))) {
            throw Lang.makeThrow(BizException.class, "删除码本分组失败!");
        }
    }

    @GetMapping("group/{group}/dictionaries")
    @Operation(summary = "分组下的数据字典列表")
    public List<Dictionary> dictionaries(@Parameter(description = "码本分组key", required = true) @PathVariable String group) {
        return dictionaryService.list(Wrappers.<Dictionary> lambdaQuery().eq(Dictionary::getGroupKey, group).orderByAsc(Dictionary::getIndex, Dictionary::getId));
    }

    @PutMapping("group/{group}/dictionary")
    @Operation(summary = "增加/编辑码本数据")
    public Dictionary saveOrUpdateDictionary(
                                             @Parameter(description = "码本分组key", required = true) @PathVariable String group,
                                             @Validated @Parameter(description = "码本数据") @RequestBody Dictionary dictionary) {
        dictionary.setGroupKey(group);
        if (dictionary.getId() != null && dictionary.getId() > 0) {
            if (dictionaryService.update(dictionary, Wrappers.<Dictionary> lambdaUpdate().eq(Dictionary::getId, dictionary.getId()))) {
                return dictionary;
            } else {
                throw Lang.makeThrow("更新码本数据失败!");
            }
        } else {
            if (dictionaryService.save(dictionary)) {
                return dictionary;
            } else {
                throw BizException.create("保存码本分组失败!");
            }
        }
    }

    @DeleteMapping("group/{group}/dictionary/{key}")
    @Operation(summary = "删除码本数据")
    @ResponseStatus(HttpStatus.OK)
    public void deleteDictionary(@Parameter(description = "码本分组key", required = true) @PathVariable String group,
                                 @Parameter(description = "码本数据key", required = true) @PathVariable String key) {
        if (!dictionaryService.remove(Wrappers.<Dictionary> lambdaQuery().eq(Dictionary::getGroupKey, group).eq(Dictionary::getKey, key))) {
            throw Lang.makeThrow("删除码本数据失败!");
        }
    }

}
