package tech.riemann.ims.utils;

import lombok.experimental.UtilityClass;

import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ThreadLocalRandom;

/**
 * @author mayong
 * @since 2024/12/12 20:39
 */
@UtilityClass
public class BarcodeUtil {

    // 用于存储每天已经生成的条码的最后三位
    private static final ConcurrentHashMap<String, Set<Integer>> dailySequences = new ConcurrentHashMap<>();

    // 日期格式化为年份后两位和一年中的某一天
    private static final SimpleDateFormat sdf = new SimpleDateFormat("yyDDD");

    /**
     * 生成一个7位数字条码，前4位表示年份后两位和一年中的某一天，后3位随机且当天不重复
     */
    public static synchronized String generateBarcode() {
        // 获取当前日期部分（简化为年份后两位和一年中的第几天）
        String datePart = sdf.format(new Date());

        // 如果当天的序列号集合不存在，则初始化一个新的集合
        dailySequences.putIfAbsent(datePart, Collections.synchronizedSet(new HashSet<>()));

        // 获取当天的序列号集合
        Set<Integer> sequencesToday = dailySequences.get(datePart);

        // 生成随机且不重复的三位数
        int sequence;
        do {
            sequence = ThreadLocalRandom.current().nextInt(0, 9999);
        } while (sequencesToday.contains(sequence));

        // 将新生成的序列号添加到当天的集合中
        sequencesToday.add(sequence);

        // 格式化序列号为3位字符串，不足前面补0
        String sequencePart = String.format("%04d", sequence);

        return datePart + sequencePart;
    }

    /**
     * 模拟新的一天开始，重置当天的序列号集合
     */
    public static void resetDailySequence() {
        dailySequences.clear();
    }
}
